@@ -20,9 +20,10 @@
 import os
 import shapefile
 import numpy as np
-from PIL import Image, ImageFont
+from PIL import Image
 import pyproj
 import logging
+import ast
 
 try:
     import configparser
@@ -32,6 +33,37 @@ except ImportError:
 logger = logging.getLogger(__name__)
 
 
+def get_resolution_from_area(area_def):
+    """Get the best resolution for an area definition."""
+    x_size = area_def.width
+    y_size = area_def.height
+    prj = Proj(area_def.crs if hasattr(area_def, 'crs') else area_def.proj_str)
+    if prj.is_latlong():
+        x_ll, y_ll = prj(area_def.area_extent[0], area_def.area_extent[1])
+        x_ur, y_ur = prj(area_def.area_extent[2], area_def.area_extent[3])
+        x_resolution = (x_ur - x_ll) / x_size
+        y_resolution = (y_ur - y_ll) / y_size
+    else:
+        x_resolution = ((area_def.area_extent[2] -
+                         area_def.area_extent[0]) /
+                        x_size)
+        y_resolution = ((area_def.area_extent[3] -
+                         area_def.area_extent[1]) /
+                        y_size)
+    res = min(x_resolution, y_resolution)
+
+    if res > 25000:
+        return "c"
+    elif res > 5000:
+        return "l"
+    elif res > 1000:
+        return "i"
+    elif res > 200:
+        return "h"
+    else:
+        return "f"
+
+
 class Proj(pyproj.Proj):
     """Wrapper around pyproj to add in 'is_latlong'."""
 
@@ -99,7 +131,7 @@ class ContourWriterBase(object):
 
     def _find_line_intercepts(self, xys, size, margins):
         """Finds intercepts of poly-line xys with image boundaries
-        offset by margins and returns an array of coordintes"""
+        offset by margins and returns an array of coordinates"""
         x_size, y_size = size
 
         def is_in_box(x_y, extents):
@@ -120,13 +152,13 @@ class ContourWriterBase(object):
         xlim1 = margins[0]
         ylim1 = margins[1]
         xlim2 = x_size - margins[0]
-        ylim2 = y_size - margins[0]
+        ylim2 = y_size - margins[1]
 
         # only consider crossing within a box a little bigger than grid
         # boundary
         search_box = (-10, x_size + 10, -10, y_size + 10)
 
-        # loop trought line steps and detect crossings
+        # loop through line steps and detect crossings
         intercepts = []
         align_left = 'LC'
         align_right = 'RC'
@@ -168,10 +200,10 @@ class ContourWriterBase(object):
         """
 
         try:
-            proj4_string = area_def.proj4_string
+            proj_def = area_def.crs if hasattr(area_def, 'crs') else area_def.proj_dict
             area_extent = area_def.area_extent
         except AttributeError:
-            proj4_string = area_def[0]
+            proj_def = area_def[0]
             area_extent = area_def[1]
 
         draw = self._get_canvas(image)
@@ -192,7 +224,7 @@ class ContourWriterBase(object):
 
         # Area and projection info
         x_size, y_size = image.size
-        prj = Proj(proj4_string)
+        prj = Proj(proj_def)
 
         x_offset = 0
         y_offset = 0
@@ -234,7 +266,7 @@ class ContourWriterBase(object):
                              lat_max - shorten_max_lat,
                              float(lat_max - lat_min) / y_size)
         # lin_lats in rather high definition so that it can be used to
-        # posituion text labels near edges of image...
+        # position text labels near edges of image...
 
         # perhaps better to find the actual length of line in pixels...
 
@@ -253,7 +285,7 @@ class ContourWriterBase(object):
 
         # lons along major lat lines (extended slightly to avoid missing the
         # end)
-        lin_lons = np.arange(lon_min, lon_max + Dlon / 5.0, Dlon / 10.0)
+        lin_lons = np.linspace(lon_min, lon_max + Dlon / 5.0, max(x_size, y_size) // 5)
 
         # MINOR LINES ######
         if not kwargs['minor_is_tick']:
@@ -298,9 +330,9 @@ class ContourWriterBase(object):
         for lon in maj_lons:
             # Draw 'minor' tick lines dlat separation along the lon
             if kwargs['minor_is_tick']:
-                tick_lons = np.arange(lon - Dlon / 20.0,
-                                      lon + Dlon / 20.0,
-                                      Dlon / 50.0)
+                tick_lons = np.linspace(lon - Dlon / 20.0,
+                                        lon + Dlon / 20.0,
+                                        5)
 
                 for lat in min_lats:
                     lonlats = [(x, lat) for x in tick_lons]
@@ -354,9 +386,9 @@ class ContourWriterBase(object):
         for lat in maj_lats:
             # Draw 'minor' tick dlon separation along the lat
             if kwargs['minor_is_tick']:
-                tick_lats = np.arange(lat - Dlat / 20.0,
-                                      lat + Dlat / 20.0,
-                                      Dlat / 50.0)
+                tick_lats = np.linspace(lat - Dlat / 20.0,
+                                        lat + Dlat / 20.0,
+                                        5)
                 for lon in min_lons:
                     lonlats = [(lon, x) for x in tick_lats]
                     index_arrays, is_reduced = \
@@ -512,17 +544,17 @@ class ContourWriterBase(object):
 
         """
         try:
-            proj4_string = area_def.proj4_string
+            proj_def = area_def.crs if hasattr(area_def, 'crs') else area_def.proj_dict
             area_extent = area_def.area_extent
         except AttributeError:
-            proj4_string = area_def[0]
+            proj_def = area_def[0]
             area_extent = area_def[1]
 
         draw = self._get_canvas(image)
 
         # Area and projection info
         x_size, y_size = image.size
-        prj = Proj(proj4_string)
+        prj = Proj(proj_def)
 
         # Calculate min and max lons and lats of interest
         lon_min, lon_max, lat_min, lat_max = _get_lon_lat_bounding_box(area_extent, x_size, y_size, prj)
@@ -617,9 +649,9 @@ class ContourWriterBase(object):
         else:
             format_string += 'L%s.shp'
 
-        if type(level) not in (list,):
-            level = range(1,level+1)
-                        
+        if not isinstance(level, list):
+            level = range(1, level + 1)
+
         for i in level:
 
             # One shapefile per level
@@ -644,22 +676,12 @@ class ContourWriterBase(object):
                 yield shape
 
     def _finalize(self, draw):
-        """Do any need finalization of the drawing
-        """
+        """Do any need finalization of the drawing."""
 
         pass
 
-    def add_overlay_from_config(self, config_file, area_def):
-        """Create and return a transparent image adding all the overlays contained in a configuration file.
-
-        :Parameters:
-            config_file : str
-                Configuration file name
-            area_def : object
-                Area Definition of the creating image
-
-        """
-
+    def _config_to_dict(self, config_file):
+        """Convert a config file to a dict."""
         config = configparser.ConfigParser()
         try:
             with open(config_file, 'r'):
@@ -673,58 +695,79 @@ class ContourWriterBase(object):
             logger.error("Error in %s", str(config_file))
             raise
 
+        SECTIONS = ['cache', 'coasts', 'rivers', 'borders', 'cities', 'grid']
+        overlays = {}
+        for section in config.sections():
+            if section in SECTIONS:
+                overlays[section] = {}
+                for option in config.options(section):
+                    val = config.get(section, option)
+                    try:
+                        overlays[section][option] = ast.literal_eval(val)
+                    except ValueError:
+                        overlays[section][option] = val
+        return overlays
+
+    def add_overlay_from_dict(self, overlays, area_def, cache_epoch=None, background=None):
+        """Create and return a transparent image adding all the overlays contained in the `overlays` dict.
+
+        :Parameters:
+            overlays : dict
+                overlays configuration
+            area_def : object
+                Area Definition of the creating image
+            cache_epoch: seconds since epoch
+                The latest time allowed for cache the cache file. If the cache file is older than this (mtime),
+                the cache should be regenerated.
+            background: pillow image instance
+                The image on which to write the overlays on. If it's None (default),
+                a new image is created, otherwise the provide background is use
+                an change *in place*.
+
+
+            The keys in `overlays` that will be taken into account are:
+            cache, coasts, rivers, borders, cities, grid
+
+            For all of them except `cache`, the items are the same as the corresponding
+            functions in pycoast, so refer to the docstrings of these functions
+            (add_coastlines, add_rivers, add_borders, add_grid, add_cities).
+            For cache, two parameters are configurable: `file` which specifies the directory
+            and the prefix of the file to save the caches decoration to
+            (for example /var/run/black_coasts_red_borders), and `regenerate` that can be
+            True or False (default) to force the overwriting of an already cached file.
+
+        """
+
         # Cache management
         cache_file = None
-        if config.has_section('cache'):
-            config_file_name, config_file_extention = \
-                os.path.splitext(config_file)
-            cache_file = (config.get('cache', 'file') + '_' +
+        if 'cache' in overlays:
+            cache_file = (overlays['cache']['file'] + '_' +
                           area_def.area_id + '.png')
 
             try:
-                configTime = os.path.getmtime(config_file)
-                cacheTime = os.path.getmtime(cache_file)
+                config_time = cache_epoch
+                cache_time = os.path.getmtime(cache_file)
                 # Cache file will be used only if it's newer than config file
-                if configTime < cacheTime:
+                if ((config_time is not None and config_time < cache_time)
+                        and not overlays['cache'].get('regenerate', False)):
                     foreground = Image.open(cache_file)
                     logger.info('Using image in cache %s', cache_file)
+                    if background is not None:
+                        background.paste(foreground, mask=foreground.split()[-1])
                     return foreground
                 else:
-                    logger.info("Cache file is not used "
-                                "because config file has changed")
+                    logger.info("Regenerating cache file.")
             except OSError:
-                logger.info("New overlay image will be saved in cache")
+                logger.info("No overlay image found, new overlay image will be saved in cache.")
 
-        x_size = area_def.x_size
-        y_size = area_def.y_size
-        foreground = Image.new('RGBA', (x_size, y_size), (0, 0, 0, 0))
-
-        # Lines (coasts, rivers, borders) management
-        prj = Proj(area_def.proj4_string)
-        if prj.is_latlong():
-            x_ll, y_ll = prj(area_def.area_extent[0], area_def.area_extent[1])
-            x_ur, y_ur = prj(area_def.area_extent[2], area_def.area_extent[3])
-            x_resolution = (x_ur - x_ll) / x_size
-            y_resolution = (y_ur - y_ll) / y_size
+        x_size = area_def.width
+        y_size = area_def.height
+        if cache_file is None and background is not None:
+            foreground = background
         else:
-            x_resolution = ((area_def.area_extent[2] -
-                             area_def.area_extent[0]) /
-                            x_size)
-            y_resolution = ((area_def.area_extent[3] -
-                             area_def.area_extent[1]) /
-                            y_size)
-        res = min(x_resolution, y_resolution)
+            foreground = Image.new('RGBA', (x_size, y_size), (0, 0, 0, 0))
 
-        if res > 25000:
-            default_resolution = "c"
-        elif res > 5000:
-            default_resolution = "l"
-        elif res > 1000:
-            default_resolution = "i"
-        elif res > 200:
-            default_resolution = "h"
-        else:
-            default_resolution = "f"
+        default_resolution = get_resolution_from_area(area_def)
 
         DEFAULT = {'level': 1,
                    'outline': 'white',
@@ -736,18 +779,9 @@ class ContourWriterBase(object):
                    'y_offset': 0,
                    'resolution': default_resolution}
 
-        SECTIONS = ['coasts', 'rivers', 'borders', 'cities']
-        overlays = {}
-
-        for section in config.sections():
-            if section in SECTIONS:
-                overlays[section] = {}
-                for option in config.options(section):
-                    overlays[section][option] = config.get(section, option)
-
         is_agg = self._draw_module == "AGG"
 
-        # Coasts
+        # Coasts, rivers, borders
         for section, fun in zip(['coasts', 'rivers', 'borders'],
                                 [self.add_coastlines,
                                  self.add_rivers,
@@ -758,13 +792,6 @@ class ContourWriterBase(object):
                 params = DEFAULT.copy()
                 params.update(overlays[section])
 
-                params['level'] = int(params['level'])
-                params['x_offset'] = float(params['x_offset'])
-                params['y_offset'] = float(params['y_offset'])
-                params['width'] = float(params['width'])
-                params['outline_opacity'] = int(params['outline_opacity'])
-                params['fill_opacity'] = int(params['fill_opacity'])
-
                 if section != "coasts":
                     params.pop('fill_opacity', None)
                     params.pop('fill', None)
@@ -795,14 +822,63 @@ class ContourWriterBase(object):
                             font_size, pt_size, outline, box_outline,
                             box_opacity)
 
+        if 'grid' in overlays:
+            lon_major = float(overlays['grid'].get('lon_major', 10.0))
+            lat_major = float(overlays['grid'].get('lat_major', 10.0))
+            lon_minor = float(overlays['grid'].get('lon_minor', 2.0))
+            lat_minor = float(overlays['grid'].get('lat_minor', 2.0))
+            font = overlays['grid'].get('font', None)
+            font_size = int(overlays['grid'].get('font_size', 10))
+
+            write_text = overlays['grid'].get('write_text', True)
+            if isinstance(write_text, str):
+                write_text = write_text.lower() in ['true', 'yes', '1', 'on']
+            outline = overlays['grid'].get('outline', 'white')
+            if isinstance(font, str):
+                if is_agg:
+                    from aggdraw import Font
+                    font = Font(outline, font, size=font_size)
+                else:
+                    from PIL.ImageFont import truetype
+                    font = truetype(font, font_size)
+            fill = overlays['grid'].get('fill', None)
+            minor_outline = overlays['grid'].get('minor_outline', 'white')
+            minor_is_tick = overlays['grid'].get('minor_is_tick', True)
+            if isinstance(minor_is_tick, str):
+                minor_is_tick = minor_is_tick.lower() in ['true', 'yes', '1']
+            lon_placement = overlays['grid'].get('lon_placement', 'tb')
+            lat_placement = overlays['grid'].get('lat_placement', 'lr')
+
+            self.add_grid(foreground, area_def, (lon_major, lat_major),
+                          (lon_minor, lat_minor),
+                          font=font, write_text=write_text, fill=fill,
+                          outline=outline, minor_outline=minor_outline,
+                          minor_is_tick=minor_is_tick,
+                          lon_placement=lon_placement,
+                          lat_placement=lat_placement)
+
         if cache_file is not None:
             try:
                 foreground.save(cache_file)
             except IOError as e:
                 logger.error("Can't save cache: %s", str(e))
-
+            if background is not None:
+                background.paste(foreground, mask=foreground.split()[-1])
         return foreground
 
+    def add_overlay_from_config(self, config_file, area_def, background=None):
+        """Create and return a transparent image adding all the overlays contained in a configuration file.
+
+        :Parameters:
+            config_file : str
+                Configuration file name
+            area_def : object
+                Area Definition of the creating image
+
+        """
+        overlays = self._config_to_dict(config_file)
+        return self.add_overlay_from_dict(overlays, area_def, os.path.getmtime(config_file), background)
+
     def add_cities(self, image, area_def, citylist, font_file, font_size,
                    ptsize, outline, box_outline, box_opacity, db_root_path=None):
         """Add cities (point and name) to a PIL image object
@@ -812,20 +888,8 @@ class ContourWriterBase(object):
             db_root_path = self.db_root_path
         if db_root_path is None:
             raise ValueError("'db_root_path' must be specified to use this method")
-
-        try:
-            proj4_string = area_def.proj4_string
-            area_extent = area_def.area_extent
-        except AttributeError:
-            proj4_string = area_def[0]
-            area_extent = area_def[1]
-
         draw = self._get_canvas(image)
 
-        # Area and projection info
-        x_size, y_size = image.size
-        prj = Proj(proj4_string)
-
         # read shape file with points
         # Sc-Kh shapefilename = os.path.join(self.db_root_path,
         # "cities_15000_alternativ.shp")
@@ -906,6 +970,8 @@ def _get_lon_lat_bounding_box(area_extent, x_size, y_size, prj):
     prev = None
     for lon in np.concatenate((lons_s1, lons_s2,
                                lons_s3[::-1], lons_s4[::-1])):
+        if not np.isfinite(lon):
+            continue
         if prev is not None:
             delta = lon - prev
             if abs(delta) > 180:
@@ -950,6 +1016,7 @@ def _get_lon_lat_bounding_box(area_extent, x_size, y_size, prj):
         lon_min = -180
         lon_max = 180
 
+    # Catch inf/1e30 or other invalid values
     if not (-180 <= lon_min <= 180):
         lon_min = -180
     if not (-180 <= lon_max <= 180):
@@ -981,16 +1048,16 @@ def _get_pixel_index(shape, area_extent, x_size, y_size, prj,
     # Handle out of bounds
     i = 0
     segments = []
-    if 1e30 in x or 1e30 in y:
+    if (x >= 1e30).any() or (y >= 1e30).any():
         # Split polygon in line segments within projection
         is_reduced = True
-        if x[0] == 1e30 or y[0] == 1e30:
+        if x[0] >= 1e30 or y[0] >= 1e30:
             in_segment = False
         else:
             in_segment = True
 
         for j in range(x.size):
-            if (x[j] == 1e30 or y[j] == 1e30):
+            if x[j] >= 1e30 or y[j] >= 1e30:
                 if in_segment:
                     segments.append((x[i:j], y[i:j]))
                     in_segment = False
