@@ -22,7 +22,7 @@ import unittest
 
 import numpy as np
 from PIL import Image, ImageFont
-
+import time
 
 def tmp(f):
     f.tmp = True
@@ -68,6 +68,7 @@ def fft_metric(data1, data2, max_value=0.1):
 gshhs_root_dir = os.path.join(os.path.dirname(__file__), 'test_data', 'gshhs')
 test_file = 'test_image.png'
 grid_file = 'test_grid.png'
+p_file_coasts = 'test_coasts_p_mode.png'
 
 
 class TestPycoast(unittest.TestCase):
@@ -76,10 +77,13 @@ class TestPycoast(unittest.TestCase):
         img = Image.new('RGB', (640, 480))
         img.save(test_file)
         img.save(grid_file)
+        img_p = Image.new('P', (640, 480))
+        img_p.save(p_file_coasts)
 
     def tearDown(self):
         os.remove(test_file)
         os.remove(grid_file)
+        os.remove(p_file_coasts)
 
 
 class TestPIL(TestPycoast):
@@ -167,6 +171,32 @@ class TestPIL(TestPycoast):
         res = np.array(img)
         self.assertTrue(fft_metric(grid_data, res), 'Writing of grid failed')
 
+    def test_grid_germ(self):
+        """Check that issue #26 is fixed."""
+        from pycoast import ContourWriterPIL
+        result_file = os.path.join(os.path.dirname(__file__), 'grid_germ.png')
+        grid_img = Image.open(result_file)
+        grid_data = np.array(grid_img)
+        img = Image.new('RGB', (1024, 1024))
+        proj4_string = \
+            '+proj=stere +ellps=bessel +lat_0=90.0 +lon_0=5.0 +lat_ts=50.0 +a=6378144.0 +b=6356759.0'
+        area_extent = [-155100.436345, -4441495.37946, 868899.563655, -3417495.37946]
+
+        area_def = (proj4_string, area_extent)
+
+        cw = ContourWriterPIL(gshhs_root_dir)
+
+        cw.add_coastlines(img, area_def, resolution='l', level=4)
+        font = ImageFont.truetype(os.path.join(os.path.dirname(__file__),
+                                               'test_data', 'DejaVuSerif.ttf'),
+                                  16)
+        cw.add_grid(img, area_def, (10.0, 10.0), (2.0, 2.0),
+                    font=font, fill='yellow', write_text=True,
+                    outline='red', minor_outline='white')
+
+        res = np.array(img)
+        self.assertTrue(fft_metric(grid_data, res), 'Writing of grid to germ failed')
+
     def test_grid_geos(self):
         from pycoast import ContourWriterPIL
         geos_img = Image.open(
@@ -359,6 +389,29 @@ class TestPIL(TestPycoast):
         self.assertTrue(
             fft_metric(grid_data, res), 'Writing of Brazil shapefiles failed')
 
+    def test_config_file_coasts_and_grid(self):
+        from pycoast import ContourWriterPIL
+        from pyresample.geometry import AreaDefinition
+        overlay_config = os.path.join(os.path.dirname(__file__),
+                                      "coasts_and_grid.ini")
+        grid_img = Image.open(os.path.join(os.path.dirname(__file__),
+                                           'grid_nh.png'))
+        grid_data = np.array(grid_img)
+        proj_dict = {'proj': 'laea', 'lat_0': 90.0, 'lon_0': 0.0,
+                     'a': 6371228.0, 'units': 'm'}
+        area_extent = (-5326849.0625, -5326849.0625,
+                       5326849.0625, 5326849.0625)
+        area_def = AreaDefinition('nh', 'nh', 'nh', proj_dict, 425, 425,
+                                  area_extent)
+
+        cw = ContourWriterPIL(gshhs_root_dir)
+        overlay = cw.add_overlay_from_config(overlay_config, area_def)
+        img = Image.new('RGB', (425, 425))
+        img.paste(overlay, mask=overlay)
+        res = np.array(img)
+        self.assertTrue(fft_metric(grid_data, res),
+                        'Writing of nh grid failed')
+
 
 class TestPILAGG(TestPycoast):
 
@@ -624,12 +677,168 @@ class TestPILAGG(TestPycoast):
         self.assertTrue(
             fft_metric(grid_data, res), 'Writing of Brazil shapefiles failed')
 
+    @unittest.skip("All kwargs are not supported, so can't create equal results")
+    def test_config_file_coasts_and_grid(self):
+        from pycoast import ContourWriterAGG
+        from pyresample.geometry import AreaDefinition
+        overlay_config = os.path.join(os.path.dirname(__file__),
+                                      "coasts_and_grid_agg.ini")
+        grid_img = Image.open(os.path.join(os.path.dirname(__file__),
+                                           'grid_nh_agg.png'))
+        grid_data = np.array(grid_img)
+        proj_dict = {'proj': 'laea', 'lat_0': 90.0, 'lon_0': 0.0,
+                     'a': 6371228.0, 'units': 'm'}
+        area_extent = (-5326849.0625, -5326849.0625,
+                       5326849.0625, 5326849.0625)
+        area_def = AreaDefinition('nh', 'nh', 'nh', proj_dict, 425, 425,
+                                  area_extent)
+
+        cw = ContourWriterAGG(gshhs_root_dir)
+        overlay = cw.add_overlay_from_config(overlay_config, area_def)
+        img = Image.new('RGB', (425, 425))
+        img.paste(overlay, mask=overlay)
+
+        res = np.array(img)
+        self.assertTrue(fft_metric(grid_data, res),
+                        'Writing of nh grid failed')
+
+    def test_coastlines_convert_to_rgba_agg(self):
+        from pycoast import ContourWriterAGG
+        proj4_string = \
+            '+proj=stere +lon_0=8.00 +lat_0=50.00 +lat_ts=50.00 +ellps=WGS84'
+        area_extent = (-3363403.31, -2291879.85, 2630596.69, 2203620.1)
+        area_def = (proj4_string, area_extent)
+
+        cw = ContourWriterAGG(gshhs_root_dir)
+        cw.add_coastlines_to_file(p_file_coasts, area_def, resolution='l', level=4)
+
+        img = Image.open(p_file_coasts)
+        image_mode = img.mode
+        img.close()
+
+        self.assertTrue(image_mode == 'RGBA', 'Conversion to RGBA failed.')
+
+
+class FakeAreaDef():
+    """A fake area definition object."""
+
+    def __init__(self, proj4_string, area_extent, x_size, y_size):
+        self.proj_str = self.proj_dict = self.crs = proj4_string
+        self.area_extent = area_extent
+        self.width = x_size
+        self.height = y_size
+        self.area_id = 'fakearea'
+
+
+class TestFromConfig(TestPycoast):
+    """Test burning overlays from a config file."""
+
+    def test_foreground(self):
+        """Test generating a transparent foreground."""
+        from pycoast import ContourWriterPIL
+        euro_img = Image.open(os.path.join(os.path.dirname(__file__),
+                                           'contours_europe_alpha.png'))
+        euro_data = np.array(euro_img)
+
+        # img = Image.new('RGB', (640, 480))
+        proj4_string = \
+            '+proj=stere +lon_0=8.00 +lat_0=50.00 +lat_ts=50.00 +ellps=WGS84'
+        area_extent = (-3363403.31, -2291879.85, 2630596.69, 2203620.1)
+        area_def = FakeAreaDef(proj4_string, area_extent, 640, 480)
+        cw = ContourWriterPIL(gshhs_root_dir)
+        config_file = os.path.join(os.path.dirname(__file__), 'test_data', 'test_config.ini')
+        img = cw.add_overlay_from_config(config_file, area_def)
+
+        res = np.array(img)
+        self.assertTrue(fft_metric(euro_data, res),
+                        'Writing of contours failed')
+
+        overlays = {'coasts': {'level': [1, 2, 3, 4], 'resolution': 'l'},
+                    'borders': {'outline': (255, 0, 0), 'resolution': 'c'},
+                    'rivers': {'outline': 'blue', 'resolution': 'c', 'level': 5}}
+
+        img = cw.add_overlay_from_dict(overlays, area_def)
+        res = np.array(img)
+        self.assertTrue(fft_metric(euro_data, res),
+                        'Writing of contours failed')
+
+    def test_cache(self):
+        """Test generating a transparent foreground and cache it."""
+        from pycoast import ContourWriterPIL
+        from tempfile import gettempdir
+        euro_img = Image.open(os.path.join(os.path.dirname(__file__),
+                                           'contours_europe_alpha.png'))
+        euro_data = np.array(euro_img)
+
+        # img = Image.new('RGB', (640, 480))
+        proj4_string = \
+            '+proj=stere +lon_0=8.00 +lat_0=50.00 +lat_ts=50.00 +ellps=WGS84'
+        area_extent = (-3363403.31, -2291879.85, 2630596.69, 2203620.1)
+        area_def = FakeAreaDef(proj4_string, area_extent, 640, 480)
+        cw = ContourWriterPIL(gshhs_root_dir)
+
+        tmp = gettempdir()
+
+        overlays = {'cache': {'file': os.path.join(tmp, 'pycoast_cache')},
+                    'coasts': {'level': 4, 'resolution': 'l'},
+                    'borders': {'outline': (255, 0, 0), 'resolution': 'c'},
+                    'rivers': {'outline': 'blue', 'resolution': 'c', 'level': 5}}
+
+        cache_filename = os.path.join(tmp, 'pycoast_cache_fakearea.png')
+        img = cw.add_overlay_from_dict(overlays, area_def)
+        res = np.array(img)
+        self.assertTrue(fft_metric(euro_data, res),
+                        'Writing of contours failed')
+        self.assertTrue(os.path.isfile(cache_filename))
+
+        current_time = time.time()
+
+        img = cw.add_overlay_from_dict(overlays, area_def, current_time)
+
+        mtime = os.path.getmtime(cache_filename)
+
+        self.assertGreater(mtime, current_time)
+        self.assertTrue(fft_metric(euro_data, res),
+                        'Writing of contours failed')
+
+        img = cw.add_overlay_from_dict(overlays, area_def, current_time)
+
+        self.assertEqual(os.path.getmtime(cache_filename), mtime)
+        self.assertTrue(fft_metric(euro_data, res),
+                        'Writing of contours failed')
+        overlays['cache']['regenerate'] = True
+        img = cw.add_overlay_from_dict(overlays, area_def)
+
+        self.assertNotEqual(os.path.getmtime(cache_filename), mtime)
+        self.assertTrue(fft_metric(euro_data, res),
+                        'Writing of contours failed')
+
+        overlays.pop('cache')
+        overlays['grid'] = {'outline': (255, 255, 255), 'outline_opacity': 175,
+                            'minor_outline': (200, 200, 200), 'minor_outline_opacity': 127,
+                            'width': 1.0, 'minor_width': 0.5, 'minor_is_tick': True,
+                            'write_text': True, 'lat_placement': 'lr', 'lon_placement': 'b'}
+        img = cw.add_overlay_from_dict(overlays, area_def)
+        os.remove(os.path.join(tmp, 'pycoast_cache_fakearea.png'))
+
+    def test_get_resolution(self):
+        """Get the automagical resolution computation."""
+        from pycoast import get_resolution_from_area
+        proj4_string = \
+            '+proj=stere +lon_0=8.00 +lat_0=50.00 +lat_ts=50.00 +ellps=WGS84'
+        area_extent = (-3363403.31, -2291879.85, 2630596.69, 2203620.1)
+        area_def = FakeAreaDef(proj4_string, area_extent, 640, 480)
+        self.assertEqual(get_resolution_from_area(area_def), 'l')
+        area_def = FakeAreaDef(proj4_string, area_extent, 6400, 4800)
+        self.assertEqual(get_resolution_from_area(area_def), 'h')
+
 
 def suite():
     loader = unittest.TestLoader()
     mysuite = unittest.TestSuite()
     mysuite.addTest(loader.loadTestsFromTestCase(TestPIL))
     mysuite.addTest(loader.loadTestsFromTestCase(TestPILAGG))
+    mysuite.addTest(loader.loadTestsFromTestCase(TestFromConfig))
 
     return mysuite
 
